from unittest import TestCase
try:
    from urllib.error import URLError
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode
    from urllib2 import URLError
try:
    from unittest.mock import MagicMock
except ImportError:
    from mock import MagicMock

from . import DummyProvider
from .. import RedirectNeeded, PaymentError

VARIANT = 'dummy-3ds'


class Payment(object):
    id = 1
    variant = VARIANT
    currency = 'USD'
    total = 100
    status = 'waiting'
    fraud_status = ''

    def get_process_url(self):
        return 'http://example.com'

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_success_url(self):
        return 'http://success.com'

    def change_status(self, new_status):
        self.status = new_status

    def change_fraud_status(self, fraud_status):
        self.fraud_status = fraud_status


class TestDummy3DSProvider(TestCase):

    def setUp(self):
        self.payment = Payment()

    def test_process_data_supports_verification_result(self):
        provider = DummyProvider()
        verification_status = 'confirmed'
        request = MagicMock()
        request.GET = {'verification_result': verification_status}
        response = provider.process_data(self.payment, request)
        self.assertEqual(self.payment.status, verification_status)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], self.payment.get_success_url())

    def test_process_data_redirects_to_success_on_payment_success(self):
        self.payment.status = 'preauth'
        provider = DummyProvider()
        request = MagicMock()
        request.GET = {}
        response = provider.process_data(self.payment, request)
        self.assertEqual(response['location'], self.payment.get_success_url())

    def test_process_data_redirects_to_failure_on_payment_failure(self):
        self.payment.status = 'reject'
        provider = DummyProvider()
        request = MagicMock()
        request.GET = {}
        response = provider.process_data(self.payment, request)
        self.assertEqual(response['location'], self.payment.get_failure_url())

    def test_provider_supports_non_3ds_transactions(self):
        provider = DummyProvider()
        data = {
            'status': 'preauth',
            'fraud_status': 'unknown',
            'gateway_response': '3ds-disabled',
            'verification_result': ''
        }
        with self.assertRaises(RedirectNeeded) as exc:
            provider.get_form(self.payment, data)
            self.assertEqual(exc.args[0], self.payment.get_success_url())

    def test_provider_raises_verification_result_needed_on_success(self):
        provider = DummyProvider()
        data = {
            'status': 'waiting',
            'fraud_status': 'unknown',
            'gateway_response': '3ds-redirect'}

        form = provider.get_form(self.payment, data)
        self.assertFalse(form.is_valid())

    def test_provider_supports_3ds_redirect(self):
        provider = DummyProvider()
        verification_result = 'confirmed'
        data = {
            'status': 'waiting',
            'fraud_status': 'unknown',
            'gateway_response': '3ds-redirect',
            'verification_result': verification_result
        }
        params = urlencode({'verification_result': verification_result})
        expected_redirect = '%s?%s' % (self.payment.get_process_url(), params)

        with self.assertRaises(RedirectNeeded) as exc:
            provider.get_form(self.payment, data)
            self.assertEqual(exc.args[0], expected_redirect)

    def test_provider_supports_gateway_failure(self):
        provider = DummyProvider()
        data = {
            'status': 'waiting',
            'fraud_status': 'unknown',
            'gateway_response': 'failure',
            'verification_result': ''
        }
        with self.assertRaises(URLError):
            provider.get_form(self.payment, data)

    def test_provider_raises_redirect_needed_on_success(self):
        provider = DummyProvider()
        data = {
            'status': 'preauth',
            'fraud_status': 'unknown',
            'gateway_response': '3ds-disabled',
            'verification_result': ''
        }
        with self.assertRaises(RedirectNeeded) as exc:
            provider.get_form(self.payment, data)
            self.assertEqual(exc.args[0], self.payment.get_success_url())

    def test_provider_raises_redirect_needed_on_failure(self):
        provider = DummyProvider()
        data = {
            'status': 'error',
            'fraud_status': 'unknown',
            'gateway_response': '3ds-disabled',
            'verification_result': ''
        }
        with self.assertRaises(RedirectNeeded) as exc:
            provider.get_form(self.payment, data)
            self.assertEqual(exc.args[0], self.payment.get_failure_url())

    def test_provider_raises_payment_error(self):
        provider = DummyProvider()
        data = {
            'status': 'preauth',
            'fraud_status': 'unknown',
            'gateway_response': 'payment-error',
            'verification_result': ''
        }
        with self.assertRaises(PaymentError):
            provider.get_form(self.payment, data)

    def test_provider_switches_payment_status_on_get_form(self):
        provider = DummyProvider()
        form = provider.get_form(self.payment, data={})
        self.assertEqual(self.payment.status, 'input')



from __future__ import unicode_literals

try:
    from django.forms.utils import flatatt
except ImportError:
    from django.forms.util import flatatt
from django.forms.widgets import Input, HiddenInput
from django.utils.html import format_html
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _


class StripeCheckoutWidget(Input):
    is_hidden = True

    def __init__(self, provider, payment, *args, **kwargs):
        attrs = kwargs.get('attrs', {})
        kwargs['attrs'] = {
            'class': 'stripe-button',
            'data-key': provider.public_key,
            'data-image': provider.image,
            'data-name': provider.name,
            'data-description': payment.description or _('Total payment'),
            # Stripe accepts cents
            'data-amount': int(payment.total * 100),
            'data-currency': payment.currency
        }
        kwargs['attrs'].update(attrs)
        super(StripeCheckoutWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(
            attrs, src="https://checkout.stripe.com/checkout.js")
        del final_attrs['id']
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = force_text(self._format_value(value))
        return format_html('<script{0}></script>', flatatt(final_attrs))


class StripeWidget(HiddenInput):

    class Media:
        js = ['https://js.stripe.com/v2/',
              'js/payments/stripe.js']

    def build_attrs(self, extra_attrs=None, **kwargs):
        extra_attrs = dict(extra_attrs or {}, id='id_stripe_token')
        return super(StripeWidget, self).build_attrs(extra_attrs, **kwargs)



from __future__ import unicode_literals

from ..forms import CreditCardPaymentForm

RESPONSE_STATUS = {
    '1': 'confirmed',
    '2': 'rejected'}


class PaymentForm(CreditCardPaymentForm):

    def clean(self):
        cleaned_data = super(PaymentForm, self).clean()

        if not self.errors:
            if not self.payment.transaction_id:
                data = {
                    'x_card_num': cleaned_data.get('number'),
                    'x_exp_date': cleaned_data.get('expiration'),
                    'x_card_code': cleaned_data.get('cvv2')}
                response = self.provider.get_payment_response(
                    self.payment, data)
                data = response.text.split('|')
                if response.ok and RESPONSE_STATUS.get(data[0], False):
                    status = RESPONSE_STATUS.get(data[0], 'error')
                    self.payment.transaction_id = data[6]
                    if status == 'confirmed':
                        self.payment.captured_amount = self.payment.total
                    self.payment.change_status(status)
                else:
                    errors = [data[3]]
                    self._errors['__all__'] = self.error_class(errors)
                    self.payment.change_status('error')
        return cleaned_data



from __future__ import unicode_literals
import time
from decimal import Decimal
from unittest import TestCase

from django.http import HttpResponse, HttpResponseForbidden
import jwt
from mock import MagicMock

from . import GoogleWalletProvider

PAYMENT_TOKEN = '5a4dae68-2715-4b1e-8bb2-2c2dbe9255f6'
SELLER_ID = 'abc123'
SELLER_SECRET = '123abc'
VARIANT = 'wallet'


JWT_DATA = {
    'iss': 'Google',
    'aud': SELLER_ID,
    'typ': 'google/payments/inapp/item/v1/postback/buy',
    'iat': int(time.time()),
    'exp': int(time.time() + 3600),
    'request': {
        'name': 'Test Order #12',
        'price': '22.50',
        'currencyCode': 'USD',
        'sellerData': PAYMENT_TOKEN},
    'response': {
        'orderId': '1234567890'}}


class Payment(object):

    id = 1
    description = 'payment'
    currency = 'USD'
    delivery = Decimal(10)
    status = 'waiting'
    tax = Decimal(10)
    token = PAYMENT_TOKEN
    total = Decimal(100)
    variant = VARIANT

    def change_status(self, status):
        self.status = status

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_process_url(self):
        return 'http://example.com'

    def get_purchased_items(self):
        return []

    def save(self):
        return self

    def get_success_url(self):
        return 'http://success.com'


class TestGoogleWalletProvider(TestCase):

    def test_process_data(self):
        """
        GoogleWalletProvider.process_data() returns a correct HTTP response
        """
        payment = Payment()
        request = MagicMock()
        request.POST = {'jwt': jwt.encode(JWT_DATA, SELLER_SECRET)}
        provider = GoogleWalletProvider(
            seller_id=SELLER_ID, seller_secret=SELLER_SECRET)
        response = provider.process_data(payment, request)
        self.assertEqual(type(response), HttpResponse)
        self.assertEqual(payment.status, 'confirmed')

    def test_incorrect_process_data(self):
        """
        GoogleWalletProvider.process_data() checks POST data
        """
        data = dict(JWT_DATA, aud='wrong seller id')
        payment = Payment()
        request = MagicMock()
        payload = jwt.encode(data, SELLER_SECRET)
        request.POST = {'jwt': payload}
        provider = GoogleWalletProvider(
            seller_id=SELLER_ID, seller_secret=SELLER_SECRET)
        response = provider.process_data(payment, request)
        self.assertEqual(type(response), HttpResponseForbidden)

    def test_provider_request_payment_token(self):
        request = MagicMock()
        request.POST = {'jwt': jwt.encode(JWT_DATA, SELLER_SECRET)}
        provider = GoogleWalletProvider(
            seller_id=SELLER_ID, seller_secret=SELLER_SECRET)
        token = provider.get_token_from_request(None, request)
        self.assertEqual(token, PAYMENT_TOKEN)

    def test_provider_invalid_request(self):
        request = MagicMock()
        request.POST = {'jwt': 'wrong jwt data'}
        provider = GoogleWalletProvider(
            seller_id=SELLER_ID, seller_secret=SELLER_SECRET)
        token = provider.get_token_from_request(None, request)
        self.assertFalse(token)

    def test_jwt_encoder(self):
        payment = Payment()
        provider = GoogleWalletProvider(
            seller_id=SELLER_ID, seller_secret=SELLER_SECRET)
        payload = provider.get_jwt_data(payment)
        data = jwt.decode(
            payload, SELLER_SECRET, audience='Google', issuer=SELLER_ID)
        self.assertEqual(data['request']['price'], '100')

    def test_form_contains_additional_media(self):
        payment = Payment()
        library = 'http://example.com/checkout/lib.js'
        provider = GoogleWalletProvider(
            seller_id=SELLER_ID, seller_secret=SELLER_SECRET, library=library)
        form = provider.get_form(payment)
        self.assertIn(library, str(form.media))



from __future__ import unicode_literals
from decimal import Decimal
from unittest import TestCase
from django.core import signing
from mock import patch, MagicMock, Mock

from . import CyberSourceProvider, AUTHENTICATE_REQUIRED, ACCEPTED, \
    TRANSACTION_SETTLED
from .. import RedirectNeeded, PurchasedItem

MERCHANT_ID = 'abcd1234'
PASSWORD = '1234abdd1234abcd'
ORG_ID = 'abc'

PROCESS_DATA = {
    'name': 'John Doe',
    'number': '371449635398431',
    'expiration_0': '5',
    'expiration_1': '2020',
    'cvv2': '1234',
    'fingerprint': 'abcd1234'}


class Payment(Mock):
    id = 1
    variant = 'cybersource'
    currency = 'USD'
    total = 100
    status = 'waiting'
    transaction_id = None
    captured_amount = 0
    message = ''

    class attrs(object):
        fingerprint_session_id = 'fake'
        merchant_defined_data = {}

    def get_process_url(self):
        return 'http://example.com'

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_success_url(self):
        return 'http://success.com'

    def change_status(self, status, message=''):
        self.status = status
        self.message = message

    def get_purchased_items(self):
        return [
            PurchasedItem(
                name='foo', quantity=Decimal('10'), price=Decimal('20'),
                currency='USD', sku='bar')]


class TestCybersourceProvider(TestCase):

    @patch('payments.cybersource.suds.client.Client', new=MagicMock())
    def setUp(self):
        self.payment = Payment()
        self.provider = CyberSourceProvider(
            merchant_id=MERCHANT_ID, password=PASSWORD, org_id=ORG_ID)

    @patch.object(CyberSourceProvider, '_make_request')
    def test_provider_raises_redirect_needed_on_success(self, mocked_request):
        transaction_id = 1234
        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = 100
        mocked_request.return_value = response
        with self.assertRaises(RedirectNeeded) as exc:
            self.provider.get_form(
                payment=self.payment, data=PROCESS_DATA)
        self.assertEqual(self.payment.status, 'confirmed')
        self.assertEqual(self.payment.captured_amount, self.payment.total)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    @patch.object(CyberSourceProvider, '_make_request')
    def test_provider_returns_form_on_3d_secure(
            self, mocked_request):
        response = MagicMock()
        response.reasonCode = AUTHENTICATE_REQUIRED
        mocked_request.return_value = response
        form = self.provider.get_form(
            payment=self.payment, data=PROCESS_DATA)
        self.assertEqual(self.payment.status, 'waiting')
        self.assertIn('PaReq', form.fields)

    @patch.object(CyberSourceProvider, '_make_request')
    def test_provider_shows_validation_error_message_response(
            self, mocked_request):
        error_message = 'The card you are trying to use was reported ' \
                        'as lost or stolen.'
        error_code = 205
        response = MagicMock()
        response.reasonCode = error_code
        mocked_request.return_value = response
        form = self.provider.get_form(
            payment=self.payment, data=PROCESS_DATA)
        self.assertEqual(form.errors['__all__'][0], error_message)

    def test_provider_shows_validation_error_message_duplicate(self):
        self.payment.transaction_id = 1
        error_message = 'This payment has already been processed.'
        form = self.provider.get_form(
            payment=self.payment, data=PROCESS_DATA)
        self.assertEqual(form.errors['__all__'][0], error_message)

    @patch.object(CyberSourceProvider, '_make_request')
    def test_provider_captures_payment(self, mocked_request):
        transaction_id = 1234
        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = TRANSACTION_SETTLED
        mocked_request.return_value = response
        self.provider.capture(self.payment)
        self.assertEqual(self.payment.status, 'confirmed')

    @patch.object(CyberSourceProvider, '_make_request')
    def test_provider_refunds_payment(self, mocked_request):
        self.payment.captured_amount = self.payment.total
        response = MagicMock()
        response.reasonCode = ACCEPTED
        mocked_request.return_value = response
        amount = self.provider.refund(self.payment)
        self.assertEqual(self.payment.total, amount)

    @patch.object(CyberSourceProvider, '_make_request')
    def test_provider_releases_payment(self, mocked_request):
        transaction_id = 123
        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = ACCEPTED
        mocked_request.return_value = response
        amount = self.provider.release(self.payment)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    @patch('payments.cybersource.redirect')
    @patch.object(CyberSourceProvider, '_make_request')
    def test_provider_redirects_on_success_captured_payment(
            self, mocked_request, mocked_redirect):
        transaction_id = 1234
        xid = 'abc'
        self.payment.attrs.xid = xid

        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = ACCEPTED
        mocked_request.return_value = response

        request = MagicMock()
        request.POST = {'MD': xid}
        request.GET = {'token': signing.dumps({
            'expiration': {'year': 2020, 'month': 9},
            'name': 'John Doe',
            'number': '371449635398431',
            'cvv2': '123'
        })}
        self.provider.process_data(self.payment, request)
        self.assertEqual(self.payment.status, 'confirmed')
        self.assertEqual(self.payment.captured_amount, self.payment.total)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    @patch('payments.cybersource.redirect')
    @patch.object(CyberSourceProvider, '_make_request')
    @patch('payments.cybersource.suds.client.Client', new=MagicMock())
    def test_provider_redirects_on_success_preauth_payment(
            self, mocked_request, mocked_redirect):
        provider = CyberSourceProvider(
            merchant_id=MERCHANT_ID, password=PASSWORD, org_id=ORG_ID,
            capture=False)
        transaction_id = 1234
        xid = 'abc'
        self.payment.attrs.xid = xid

        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = ACCEPTED
        mocked_request.return_value = response

        request = MagicMock()
        request.POST = {'MD': xid}
        request.GET = {'token': signing.dumps({
            'expiration': {'year': 2020, 'month': 9},
            'name': 'John Doe',
            'number': '371449635398431',
            'cvv2': '123'
        })}
        provider.process_data(self.payment, request)
        self.assertEqual(self.payment.status, 'preauth')
        self.assertEqual(self.payment.captured_amount, 0)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    @patch('payments.cybersource.redirect')
    @patch.object(CyberSourceProvider, '_make_request')
    @patch('payments.cybersource.suds.client.Client', new=MagicMock())
    def test_provider_redirects_on_failure(
            self, mocked_request, mocked_redirect):
        transaction_id = 1234
        xid = 'abc'
        self.payment.attrs.xid = xid

        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = 'test code'
        mocked_request.return_value = response

        request = MagicMock()
        request.POST = {'MD': xid}
        request.GET = {'token': signing.dumps({
            'expiration': {'year': 2020, 'month': 9},
            'name': 'John Doe',
            'number': '371449635398431',
            'cvv2': '123'
        })}
        self.provider.process_data(self.payment, request)
        self.assertEqual(self.payment.status, 'error')
        self.assertEqual(self.payment.captured_amount, 0)
        self.assertEqual(self.payment.transaction_id, transaction_id)



from __future__ import unicode_literals
import datetime
import os.path

from django.core import signing
from django.shortcuts import redirect
from django.utils.translation import ugettext as _
import suds.client
from suds.sax.element import Element
from suds.sudsobject import Object
import suds.wsse

from .forms import PaymentForm
from .. import ExternalPostNeeded, PaymentError, RedirectNeeded
from ..core import BasicProvider, get_credit_card_issuer
from ..forms import PaymentForm as BaseForm


ACCEPTED = 100
TRANSACTION_SETTLED = 238
TRANSACTION_REVERSED = 237
AUTHENTICATE_REQUIRED = 475

FRAUD_MANAGER_REVIEW = 480
FRAUD_MANAGER_REJECT = 481
FRAUD_SCORE_EXCEEDS_THRESHOLD = 400

# Soft Decline
ADDRESS_VERIFICATION_SERVICE_FAIL = 200
CARD_VERIFICATION_NUMBER_FAIL = 230
SMART_AUTHORIZATION_FAIL = 520

WSDL_PATH_TEST = 'xml/CyberSourceTransaction_1.101.test.wsdl'
WSDL_PATH = 'xml/CyberSourceTransaction_1.101.wsdl'


class CyberSourceProvider(BasicProvider):
    '''CyberSource payment provider
    '''

    fingerprint_url = 'https://h.online-metrix.net/fp/'

    def __init__(self, *args, **kwargs):
        self.merchant_id = kwargs.pop('merchant_id')
        self.password = kwargs.pop('password')
        local_path = os.path.dirname(__file__)
        if os.path.sep != '/':
            # ugly hack for urllib and Windows
            local_path = local_path.replace(os.path.sep, '/')
        if not local_path.startswith('/'):
            # windows paths don't start with '/'
            local_path = '/%s' % (local_path,)
        if kwargs.pop('sandbox', True):
            wsdl_path = 'file://%s/%s' % (local_path, WSDL_PATH_TEST)
            self.endpoint = (
                'https://ics2wstest.ic3.com/commerce/1.x/transactionProcessor')
        else:
            wsdl_path = 'file://%s/%s' % (local_path, WSDL_PATH)
            self.endpoint = (
                'https://ics2ws.ic3.com/commerce/1.x/transactionProcessor')
        self.client = suds.client.Client(wsdl_path)
        if 'fingerprint_url' in kwargs:
            self.fingerprint_url = kwargs.pop('fingerprint_url')
        self.org_id = kwargs.pop('org_id', None)
        security_header = suds.wsse.Security()
        security_token = suds.wsse.UsernameToken(
            username=self.merchant_id,
            password=self.password)
        security_header.tokens.append(security_token)
        self.client.set_options(soapheaders=[security_header.xml()])
        super(CyberSourceProvider, self).__init__(*args, **kwargs)

    def get_form(self, payment, data=None):
        if payment.status == 'waiting':
            payment.change_status('input')
        form = PaymentForm(data, provider=self, payment=payment)
        try:
            if form.is_valid():
                raise RedirectNeeded(payment.get_success_url())
        except ExternalPostNeeded as e:
            return e.args[0]
        return form

    def _change_status_to_confirmed(self, payment):
        if self._capture:
            payment.captured_amount = payment.total
            payment.change_status('confirmed')
        else:
            payment.change_status('preauth')

    def _set_proper_payment_status_from_reason_code(self, payment, reason_code):
        if reason_code == ACCEPTED:
            payment.change_fraud_status('accept', commit=False)
            self._change_status_to_confirmed(payment)
        elif reason_code == FRAUD_MANAGER_REVIEW:
            payment.change_fraud_status(
                'review', _(
                    'The order is marked for review by Decision Manager'),
                commit=False)
            self._change_status_to_confirmed(payment)
        elif reason_code == FRAUD_MANAGER_REJECT:
            payment.change_fraud_status(
                'reject', _('The order has been rejected by Decision Manager'),
                commit=False)
            self._change_status_to_confirmed(payment)
        elif reason_code == FRAUD_SCORE_EXCEEDS_THRESHOLD:
            payment.change_fraud_status(
                'reject', _('Fraud score exceeds threshold.'), commit=False)
            self._change_status_to_confirmed(payment)
        elif reason_code == SMART_AUTHORIZATION_FAIL:
            payment.change_fraud_status(
                'reject', _('CyberSource Smart Authorization failed.'),
                commit=False)
            self._change_status_to_confirmed(payment)
        elif reason_code == CARD_VERIFICATION_NUMBER_FAIL:
            payment.change_fraud_status(
                'reject', _('Card verification number (CVN) did not match.'),
                commit=False)
            self._change_status_to_confirmed(payment)
        elif reason_code == ADDRESS_VERIFICATION_SERVICE_FAIL:
            payment.change_fraud_status(
                'reject', _(
                    'CyberSource Address Verification Service failed.'),
                commit=False)
            self._change_status_to_confirmed(payment)
        else:
            error = self._get_error_message(reason_code)
            payment.change_status('error', message=error)
            raise PaymentError(error)

    def charge(self, payment, data):
        if self._capture:
            params = self._prepare_sale(payment, data)
        else:
            params = self._prepare_preauth(payment, data)
        response = self._make_request(payment, params)
        payment.attrs.capture = self._capture
        payment.transaction_id = response.requestID
        if response.reasonCode == AUTHENTICATE_REQUIRED:
            xid = response.payerAuthEnrollReply.xid
            payment.attrs.xid = xid
            payment.change_status(
                'waiting',
                message=_('3-D Secure verification in progress'))
            action = response.payerAuthEnrollReply.acsURL
            cc_data = dict(data)
            expiration = cc_data.pop('expiration')
            cc_data['expiration'] = {
                'month': expiration.month,
                'year': expiration.year}
            cc_data = signing.dumps(cc_data)
            payload = {
                'PaReq': response.payerAuthEnrollReply.paReq,
                'TermUrl': self.get_return_url(payment, {'token': cc_data}),
                'MD': xid}
            form = BaseForm(data=payload, action=action, autosubmit=True)
            raise ExternalPostNeeded(form)
        else:
            self._set_proper_payment_status_from_reason_code(
                payment, response.reasonCode)

    def capture(self, payment, amount=None):
        if amount is None:
            amount = payment.total
        params = self._prepare_capture(payment, amount=amount)
        response = self._make_request(payment, params)
        if response.reasonCode == ACCEPTED:
            payment.transaction_id = response.requestID
        elif response.reasonCode == TRANSACTION_SETTLED:
                payment.change_status('confirmed')
        else:
            payment.save()
            error = self._get_error_message(response.reasonCode)
            raise PaymentError(error)
        return amount

    def release(self, payment):
        params = self._prepare_release(payment)
        response = self._make_request(payment, params)
        if response.reasonCode == ACCEPTED:
            payment.transaction_id = response.requestID
        elif response.reasonCode != TRANSACTION_REVERSED:
            payment.save()
            error = self._get_error_message(response.reasonCode)
            raise PaymentError(error)

    def refund(self, payment, amount=None):
        if amount is None:
            amount = payment.captured_amount
        params = self._prepare_refund(payment, amount=amount)
        response = self._make_request(payment, params)
        payment.save()
        if response.reasonCode != ACCEPTED:
            error = self._get_error_message(response.reasonCode)
            raise PaymentError(error)
        return amount

    def _get_error_message(self, code):
        if code in [221, 222, 700, 701, 702, 703]:
            return _(
                'Our bank has flagged your transaction as unusually suspicious. Please contact us to resolve this issue.')  # noqa
        elif code in [201, 203, 209]:
            return _(
                'Your bank has declined the transaction. No additional information was provided.')  # noqa
        elif code == 202:
            return _(
                'The card has either expired or you have entered an incorrect expiration date.')  # noqa
        elif code in [204, 210, 251]:
            return _(
                'There are insufficient funds on your card or it has reached its credit limit.')  # noqa
        elif code == 205:
            return _(
                'The card you are trying to use was reported as lost or stolen.')  # noqa
        elif code == 208:
            return _(
                'Your card is either inactive or it does not permit online payments. Please contact your bank to resolve this issue.')  # noqa
        elif code == 211:
            return _(
                'Your bank has declined the transaction. Please check the verification number of your card and retry.')  # noqa
        elif code == 231:
            return _(
                'Your bank has declined the transaction. Please make sure the card number you have entered is correct and retry.')  # noqa
        elif code in [232, 240]:
            return _(
                'We are sorry but our bank cannot handle the card type you are using.')  # noqa
        elif code in [450, 451, 452, 453, 454, 455,
                      456, 457, 458, 459, 460, 461]:
            return _(
                'We were unable to verify your address. Please make sure the address you entered is correct and retry.')  # noqa
        else:
            return _(
                'We were unable to complete the transaction. Please try again later.')  # noqa

    def _get_params_for_new_payment(self, payment):
        params = {
            'merchantID': self.merchant_id,
            'merchantReferenceCode': payment.id,
        }
        try:
            fingerprint_id = payment.attrs.fingerprint_session_id
        except KeyError:
            pass
        else:
            params['deviceFingerprintID'] = fingerprint_id
        merchant_defined_data = self._prepare_merchant_defined_data(payment)
        if merchant_defined_data:
            params['merchantDefinedData'] = merchant_defined_data
        return params

    def _make_request(self, payment, params):
        response = self.client.service.runTransaction(**params)
        payment.attrs.last_response = self._serialize_response(response)
        return response

    def _prepare_payer_auth_validation_check(self, payment, card_data,
                                             pa_response):
        check_service = self.client.factory.create(
            'data:PayerAuthValidateService')
        check_service._run = 'true'
        check_service.signedPARes = pa_response
        params = self._get_params_for_new_payment(payment)
        params['payerAuthValidateService'] = check_service
        if payment.attrs.capture:
            service = self.client.factory.create('data:CCCreditService')
            service._run = 'true'
            params['ccCreditService'] = service
        else:
            service = self.client.factory.create('data:CCAuthService')
            service._run = 'true'
            params['ccAuthService'] = service
        params.update({
            'billTo': self._prepare_billing_data(payment),
            'card': self._prepare_card_data(card_data),
            'item': self._prepare_items(payment),
            'purchaseTotals': self._prepare_totals(payment)})
        return params

    def _prepare_sale(self, payment, card_data):
        service = self.client.factory.create('data:CCCreditService')
        service._run = 'true'
        check_service = self.client.factory.create(
            'data:PayerAuthEnrollService')
        check_service._run = 'true'
        params = self._get_params_for_new_payment(payment)
        params.update({
            'ccCreditService': service,
            'payerAuthEnrollService': check_service,
            'billTo': self._prepare_billing_data(payment),
            'card': self._prepare_card_data(card_data),
            'item': self._prepare_items(payment),
            'purchaseTotals': self._prepare_totals(payment)})
        return params

    def _prepare_preauth(self, payment, card_data):
        service = self.client.factory.create('data:CCAuthService')
        service._run = 'true'
        check_service = self.client.factory.create(
            'data:PayerAuthEnrollService')
        check_service._run = 'true'
        params = self._get_params_for_new_payment(payment)
        params.update({
            'ccAuthService': service,
            'payerAuthEnrollService': check_service,
            'billTo': self._prepare_billing_data(payment),
            'card': self._prepare_card_data(card_data),
            'item': self._prepare_items(payment),
            'purchaseTotals': self._prepare_totals(payment)})
        return params

    def _prepare_capture(self, payment, amount=None):
        service = self.client.factory.create('data:CCCaptureService')
        service._run = 'true'
        service.authRequestID = payment.transaction_id
        params = {
            'merchantID': self.merchant_id,
            'merchantReferenceCode': payment.id,
            'ccCaptureService': service,
            'purchaseTotals': self._prepare_totals(payment, amount=amount)}
        return params

    def _prepare_release(self, payment):
        service = self.client.factory.create('data:CCAuthReversalService')
        service._run = 'true'
        service.authRequestID = payment.transaction_id
        params = {
            'merchantID': self.merchant_id,
            'merchantReferenceCode': payment.id,
            'ccAuthReversalService': service,
            'purchaseTotals': self._prepare_totals(payment)}
        return params

    def _prepare_refund(self, payment, amount=None):
        service = self.client.factory.create('data:CCCreditService')
        service._run = 'true'
        service.captureRequestID = payment.transaction_id
        params = {
            'merchantID': self.merchant_id,
            'merchantReferenceCode': payment.id,
            'ccCreditService': service,
            'purchaseTotals': self._prepare_totals(payment, amount=amount)}
        return params

    def _prepare_card_type(self, card_number):
        card_type, card_name = get_credit_card_issuer(card_number)
        if card_type == 'visa':
            return '001'
        elif card_type == 'mastercard':
            return '002'
        elif card_type == 'amex':
            return '003'
        elif card_type == 'jcb':
            return '004'
        elif card_type == 'maestro':
            return '042'

    def _prepare_card_data(self, data):
        card = self.client.factory.create('data:Card')
        card.fullName = data['name']
        card.accountNumber = data['number']
        card.expirationMonth = data['expiration'].month
        card.expirationYear = data['expiration'].year
        card.cvNumber = data['cvv2']
        card.cardType = self._prepare_card_type(data['number'])
        return card

    def _prepare_billing_data(self, payment):
        billing = self.client.factory.create('data:BillTo')
        billing.firstName = payment.billing_first_name
        billing.lastName = payment.billing_last_name
        billing.street1 = payment.billing_address_1
        billing.street2 = payment.billing_address_2
        billing.city = payment.billing_city
        billing.postalCode = payment.billing_postcode
        billing.country = payment.billing_country_code
        billing.state = payment.billing_country_area
        billing.email = payment.billing_email
        billing.ipAddress = payment.customer_ip_address
        return billing

    def _prepare_items(self, payment):
        items = []
        for i, item in enumerate(payment.get_purchased_items()):
            purchased = self.client.factory.create('data:Item')
            purchased._id = i
            purchased.unitPrice = str(item.price)
            purchased.quantity = str(item.quantity)
            purchased.productName = item.name
            purchased.productSKU = item.sku
            items.append(purchased)
        return items

    def _prepare_merchant_defined_data(self, payment):
        try:
            merchant_defined_data = payment.attrs.merchant_defined_data
        except KeyError:
            return
        else:
            data = self.client.factory.create('data:MerchantDefinedData')
            for i, value in merchant_defined_data.items():
                field = self.client.factory.create('data:MDDField')
                field._id = int(i)
                field.value = value
                data.mddField.append(field)
            return data

    def _prepare_totals(self, payment, amount=None):
        totals = self.client.factory.create('data:PurchaseTotals')
        totals.currency = payment.currency
        if amount is None:
            totals.grandTotalAmount = str(payment.total)
            totals.freightAmount = str(payment.delivery)
        else:
            totals.grandTotalAmount = str(amount)
        return totals

    def _serialize_response(self, response):
        if isinstance(response, (Element, Object)):
            response = dict(response)
            for k, v in response.items():
                response[k] = self._serialize_response(v)
        return response

    def process_data(self, payment, request):
        xid = request.POST.get('MD')
        if xid != payment.attrs.xid:
            return redirect(payment.get_failure_url())
        if payment.status in ['confirmed', 'preauth']:
            return redirect(payment.get_success_url())
        cc_data = request.GET.get('token')
        try:
            cc_data = signing.loads(cc_data)
        except:
            return redirect(payment.get_failure_url())
        else:
            expiration = cc_data['expiration']
            cc_data['expiration'] = datetime.date(
                expiration['year'], expiration['month'], 1)
        params = self._prepare_payer_auth_validation_check(
            payment, cc_data, request.POST.get('PaRes'))
        response = self._make_request(payment, params)
        payment.transaction_id = response.requestID
        try:
            self._set_proper_payment_status_from_reason_code(
                payment, response.reasonCode)
        except PaymentError as e:
            pass
        if payment.status in ['confirmed', 'preauth']:
            return redirect(payment.get_success_url())
        else:
            return redirect(payment.get_failure_url())



import re

from django.template.loader import render_to_string
from django.forms.widgets import TextInput, MultiWidget


class CreditCardNumberWidget(TextInput):

    def render(self, name, value, attrs=None):
        if value:
            value = re.sub('[\s-]', '', value)
            if len(value) == 16:
                value = ' '.join([value[0:4], value[4:8],
                                  value[8:12], value[12:16]])
            elif len(value) == 15:
                value = ' '.join([value[0:4], value[4:10], value[10:15]])
            elif len(value) == 14:
                value = ' '.join([value[0:4], value[4:10], value[10:14]])
        return super(CreditCardNumberWidget, self).render(name, value, attrs)


# Credit Card Expiry Fields from:
# http://www.djangosnippets.org/snippets/907/
class CreditCardExpiryWidget(MultiWidget):
    """MultiWidget for representing credit card expiry date."""
    def decompress(self, value):
        if value:
            return [value.month, value.year]
        else:
            return [None, None]

    def format_output(self, rendered_widgets):
        ctx = {'month': rendered_widgets[0], 'year': rendered_widgets[1]}
        return render_to_string('payments/credit_card_expiry_widget.html', ctx)



from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext as _
import stripe

from .widgets import StripeCheckoutWidget, StripeWidget
from .. import RedirectNeeded
from ..forms import PaymentForm as BasePaymentForm, CreditCardPaymentFormWithName
from ..models import FRAUD_CHOICES


class StripeFormMixin(object):

    charge = None

    def _handle_potentially_fraudulent_charge(self, charge, commit=True):
        fraud_details = charge['fraud_details']
        if fraud_details.get('stripe_report', None) == 'fraudulent':
            reject_fraud_choice = FRAUD_CHOICES[2][0]
            self.payment.change_fraud_status(
                reject_fraud_choice, commit=commit)
        else:
            accept_fraud_choice = FRAUD_CHOICES[1][0]
            self.payment.change_fraud_status(
                accept_fraud_choice, commit=commit)

    def clean(self):
        data = self.cleaned_data

        if not self.errors:
            if not self.payment.transaction_id:
                stripe.api_key = self.provider.secret_key
                try:
                    self.charge = stripe.Charge.create(
                        capture=False,
                        amount=int(self.payment.total * 100),
                        currency=self.payment.currency,
                        card=data['stripeToken'],
                        description='%s %s' % (
                            self.payment.billing_last_name,
                            self.payment.billing_first_name))
                except stripe.CardError as e:
                    # Making sure we retrieve the charge
                    charge_id = e.json_body['error']['charge']
                    self.charge = stripe.Charge.retrieve(charge_id)
                    # Checking if the charge was fraudulent
                    self._handle_potentially_fraudulent_charge(
                        self.charge, commit=False)
                    # The card has been declined
                    self._errors['__all__'] = self.error_class([str(e)])
                    self.payment.change_status('error', str(e))
            else:
                msg = _('This payment has already been processed.')
                self._errors['__all__'] = self.error_class([msg])

        return data

    def save(self):
        self.payment.transaction_id = self.charge.id
        self.payment.attrs.charge = stripe.util.json.dumps(self.charge)
        self.payment.change_status('preauth')
        if self.provider._capture:
            self.payment.capture()
        # Make sure we store the info of the charge being marked as fraudulent
        self._handle_potentially_fraudulent_charge(self.charge)


class ModalPaymentForm(StripeFormMixin, BasePaymentForm):

    def __init__(self, *args, **kwargs):
        super(StripeFormMixin, self).__init__(hidden_inputs=False, *args, **kwargs)
        widget = StripeCheckoutWidget(provider=self.provider, payment=self.payment)
        self.fields['stripeToken'] = forms.CharField(widget=widget)
        if self.is_bound and not self.data.get('stripeToken'):
            self.payment.change_status('rejected')
            raise RedirectNeeded(self.payment.get_failure_url())


class PaymentForm(StripeFormMixin, CreditCardPaymentFormWithName):

    stripeToken = forms.CharField(widget=StripeWidget())

    def __init__(self, *args, **kwargs):
        super(PaymentForm, self).__init__(*args, **kwargs)
        stripe_attrs = self.fields['stripeToken'].widget.attrs
        stripe_attrs['data-publishable-key'] = self.provider.public_key
        stripe_attrs['data-address-line1'] = self.payment.billing_address_1
        stripe_attrs['data-address-line2'] = self.payment.billing_address_2
        stripe_attrs['data-address-city'] = self.payment.billing_city
        stripe_attrs['data-address-state'] = self.payment.billing_country_area
        stripe_attrs['data-address-zip'] = self.payment.billing_postcode
        stripe_attrs['data-address-country'] = self.payment.billing_country_code



from __future__ import unicode_literals
from unittest import TestCase
from mock import patch, MagicMock, Mock

from . import SagepayProvider


VENDOR = 'abcd1234'
ENCRYPTION_KEY = '1234abdd1234abcd'


class Payment(Mock):
    id = 1
    variant = 'sagepay'
    currency = 'USD'
    total = 100
    status = 'waiting'
    transaction_id = None
    captured_amount = 0
    billing_first_name = 'John'

    def get_process_url(self):
        return 'http://example.com'

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_success_url(self):
        return 'http://success.com'

    def change_status(self, status):
        self.status = status


class TestSagepayProvider(TestCase):

    def setUp(self):
        self.payment = Payment()
        self.provider = SagepayProvider(
            vendor=VENDOR, encryption_key=ENCRYPTION_KEY)

    @patch('payments.sagepay.redirect')
    def test_provider_raises_redirect_needed_on_success(self, mocked_redirect):
        data = {'Status': 'OK'}
        data = "&".join(u"%s=%s" % kv for kv in data.items())
        with patch.object(SagepayProvider, 'aes_dec', return_value=data):
            self.provider.process_data(self.payment, MagicMock())
            self.assertEqual(self.payment.status, 'confirmed')
            self.assertEqual(self.payment.captured_amount, self.payment.total)

    @patch('payments.sagepay.redirect')
    def test_provider_raises_redirect_needed_on_failure(self, mocked_redirect):
        data = {'Status': ''}
        data = "&".join(u"%s=%s" % kv for kv in data.items())
        with patch.object(SagepayProvider, 'aes_dec', return_value=data):
            self.provider.process_data(self.payment, MagicMock())
            self.assertEqual(self.payment.status, 'rejected')
            self.assertEqual(self.payment.captured_amount, 0)

    def test_provider_encrypts_data(self):
        data = self.provider.get_hidden_fields(self.payment)
        decrypted_data = self.provider.aes_dec(data['Crypt'])
        self.assertIn(self.payment.billing_first_name, str(decrypted_data))

    def test_encrypt_method_returns_valid_data(self):
        encrypted = self.provider.aes_enc('mirumee')
        self.assertEqual(encrypted, b'@e63c293672f50b9c8e291831facb4e4f')



from __future__ import unicode_literals
from unittest import TestCase
from mock import patch, MagicMock, Mock

from . import BraintreeProvider
from .. import RedirectNeeded


MERCHANT_ID = 'test11'
PUBLIC_KEY = 'abcd1234'
PRIVATE_KEY = '1234abcd'

PROCESS_DATA = {
    'name': 'John Doe',
    'number': '371449635398431',
    'expiration_0': '5',
    'expiration_1': '2020',
    'cvv2': '1234'}


class Payment(Mock):
    id = 1
    variant = 'braintree'
    currency = 'USD'
    total = 100
    status = 'waiting'
    transaction_id = None
    captured_amount = 0

    def get_process_url(self):
        return 'http://example.com'

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_success_url(self):
        return 'http://success.com'

    def change_status(self, status):
        self.status = status


class TestBraintreeProvider(TestCase):

    def setUp(self):
        self.payment = Payment()

    def test_provider_redirects_to_success_on_payment_success(self):
        provider = BraintreeProvider(MERCHANT_ID, PUBLIC_KEY, PRIVATE_KEY)
        transaction_id = '12345'
        with patch('braintree.Transaction'):
            with patch('braintree.Transaction.sale') as mocked_sale:
                sale = MagicMock()
                sale.is_success = True
                sale.transaction.id = transaction_id
                mocked_sale.return_value = sale
                with self.assertRaises(RedirectNeeded) as exc:
                    provider.get_form(self.payment, data=PROCESS_DATA)
                    url = exc.args[0]
                    self.assertEqual(url, self.payment.get_success_url())
        self.assertEqual(self.payment.status, 'confirmed')
        self.assertEqual(self.payment.captured_amount, self.payment.total)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    def test_provider_shows_validation_error_message(self):
        provider = BraintreeProvider(MERCHANT_ID, PUBLIC_KEY, PRIVATE_KEY)
        error_msg = 'error message'
        with patch('braintree.Transaction'):
            with patch('braintree.Transaction.sale') as mocked_sale:
                sale = MagicMock()
                sale.is_success = False
                sale.message = error_msg
                mocked_sale.return_value = sale
                form = provider.get_form(self.payment, data=PROCESS_DATA)
                self.assertEqual(form.errors['__all__'][0], error_msg)
        self.assertEqual(self.payment.status, 'error')
        self.assertEqual(self.payment.captured_amount, 0)



from __future__ import unicode_literals
from collections import OrderedDict

import hashlib
import hmac
import json
import time

from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, HttpResponseForbidden
import requests

from ..core import BasicProvider


class CoinbaseProvider(BasicProvider):

    _method = 'get'
    api_url = 'https://api.%(endpoint)s/v1/buttons'
    checkout_url = 'https://%(endpoint)s/checkouts'

    def __init__(self, *args, **kwargs):
        self.endpoint = kwargs.pop(
            'endpoint', 'sandbox.coinbase.com')
        self.key = kwargs.pop('key')
        self.secret = kwargs.pop('secret')
        super(CoinbaseProvider, self).__init__(*args, **kwargs)
        if not self._capture:
            raise ImproperlyConfigured(
                'Coinbase does not support pre-authorization.')

    def get_custom_token(self, payment):
        value = 'coinbase-%s-%s' % (payment.token, self.key)
        return hashlib.md5(value.encode('utf-8')).hexdigest()

    def get_checkout_code(self, payment):
        api_url = self.api_url % {'endpoint': self.endpoint}
        button_data = {
            'name': payment.description,
            'price_string': str(payment.total),
            'price_currency_iso': payment.currency,
            'callback_url': self.get_return_url(payment),
            'success_url': payment.get_success_url(),
            'cancel_url': payment.get_failure_url(),
            'custom': self.get_custom_token(payment)}
        # ordered dict for stable JSON output
        data = {'button': OrderedDict(sorted(button_data.items()))}
        nonce = int(time.time() * 1e6)
        message = str(nonce) + api_url + json.dumps(data)
        signature = hmac.new(self.secret.encode(), message.encode(),
                             hashlib.sha256).hexdigest()
        headers = {
            'ACCESS_KEY': self.key,
            'ACCESS_SIGNATURE': signature,
            'ACCESS_NONCE': nonce,
            'Accept': 'application/json'}
        response = requests.post(
            api_url, data=json.dumps(data), headers=headers)

        response.raise_for_status()
        results = response.json()
        return results['button']['code']

    def get_action(self, payment):
        checkout_url = self.checkout_url % {'endpoint': self.endpoint}
        return '%s/%s' % (checkout_url, self.get_checkout_code(payment))

    def get_hidden_fields(self, payment):
        return {}

    def process_data(self, payment, request):
        try:
            results = json.loads(request.body)
        except (ValueError, TypeError):
            return HttpResponseForbidden('FAILED')

        if results['order']['custom'] != self.get_custom_token(payment):
            return HttpResponseForbidden('FAILED')

        if payment.status == 'waiting':
            payment.transaction_id = results['order']['transaction']['id']
            payment.change_status('confirmed')
            payment.save()
        return HttpResponse('OK')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from contextlib import contextmanager

from mock import patch, Mock
from unittest import TestCase
import stripe

from . import StripeProvider
from .. import RedirectNeeded


SECRET_KEY = '1234abcd'
PUBLIC_KEY = 'abcd1234'


class Payment(Mock):

    id = 1
    description = 'payment'
    currency = 'USD'
    delivery = 10
    status = 'waiting'
    message = None
    tax = 10
    total = 100
    captured_amount = 0
    transaction_id = None

    def change_status(self, status, message=''):
        self.status = status
        self.message = message

    def change_fraud_status(self, status, message='', commit=True):
        self.fraud_status = status
        self.fraud_message = message

    def capture(self, amount=None):
        amount = amount or self.total
        self.captured_amount = amount
        self.change_status('confirmed')

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_process_url(self):
        return 'http://example.com'

    def get_purchased_items(self):
        return []

    def get_success_url(self):
        return 'http://success.com'


@contextmanager
def mock_stripe_Charge_create(error_msg=None):
    json_body = {
        'error': {
            'charge': 'charge_id'
        }
    }
    with patch('stripe.Charge.create') as mocked_charge_create:
        if error_msg:
            mocked_charge_create.side_effect = stripe.CardError(
                error_msg, param=None, code=None, json_body=json_body)
        else:
            mocked_charge_create.side_effect = lambda **kwargs: {}
        yield mocked_charge_create


@contextmanager
def mock_stripe_Charge_retrieve(fraudulent=False):
    with patch('stripe.Charge.retrieve') as mocked_charge_retrieve:
        fraud_details = {
            'stripe_report': None
        }
        if fraudulent:
            fraud_details['stripe_report'] = 'fraudulent'
        mocked_charge_retrieve.side_effect = lambda charge_id: {
            'id': charge_id,
            'fraud_details': fraud_details
        }
        yield mocked_charge_retrieve


class TestStripeProvider(TestCase):

    def test_form_contains_stripe_script(self):
        payment = Payment()
        store_name = 'Test store'
        provider = StripeProvider(
            name=store_name,
            secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
        form = provider.get_form(payment)
        self.assertTrue(
            '<script class="stripe-button" data-amount="10000" '
            'data-currency="USD" data-description="payment" data-image="" '
            'data-key="%s" data-name="%s" '
            'src="https://checkout.stripe.com/checkout.js"></script>' % (
                PUBLIC_KEY, store_name)
            in str(form))

    def test_provider_raises_redirect_needed_when_token_does_not_exist(self):
        payment = Payment()
        provider = StripeProvider(
            name='Example.com store',
            secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
        data = {}
        with self.assertRaises(RedirectNeeded) as exc:
            provider.get_form(payment, data)
            self.assertEqual(exc.args[0], payment.get_failure_url())
        self.assertEqual(payment.status, 'rejected')

    def test_provider_raises_redirect_needed_on_success(self):
        payment = Payment()
        provider = StripeProvider(
            name='Example.com store',
            secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
        data = {'stripeToken': 'abcd'}
        with patch('stripe.util.json.dumps'):
            with patch('stripe.Charge.create'):
                with self.assertRaises(RedirectNeeded) as exc:
                    provider.get_form(payment, data)
                    self.assertEqual(exc.args[0], payment.get_success_url())
        self.assertEqual(payment.status, 'confirmed')
        self.assertEqual(payment.captured_amount, payment.total)

    def test_provider_shows_validation_error_message(self):
        error_msg = 'Error message'

        payment = Payment()
        provider = StripeProvider(
            name='Example.com store',
            secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
        data = {'stripeToken': 'abcd'}
        with mock_stripe_Charge_create(error_msg=error_msg):
            with mock_stripe_Charge_retrieve():
                form = provider.get_form(payment, data=data)
                self.assertEqual(form.errors['__all__'][0], error_msg)
        self.assertEqual(payment.status, 'error')
        self.assertEqual(payment.message, error_msg)
        self.assertEqual(payment.captured_amount, 0)

    def test_provider_set_fraudulent_status(self):
        error_msg = 'Error message'
        payment = Payment()
        provider = StripeProvider(
            name='Example.com store',
            secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
        data = {'stripeToken': 'abcd'}
        with mock_stripe_Charge_create(error_msg=error_msg):
            with mock_stripe_Charge_retrieve(fraudulent=True):
                provider.get_form(payment, data=data)
        self.assertEqual(payment.status, 'error')
        self.assertEqual(payment.fraud_status, 'reject')
        self.assertEqual(payment.captured_amount, 0)

    def test_provider_detect_already_processed_payment(self):
        payment = Payment()
        payment.transaction_id = 'existing_transaction_id'
        provider = StripeProvider(
            name='Example.com store',
            secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
        data = {'stripeToken': 'abcd'}
        with mock_stripe_Charge_create():
            with mock_stripe_Charge_retrieve():
                form = provider.get_form(payment, data=data)
                msg = 'This payment has already been processed.'
                self.assertEqual(form.errors['__all__'][0], msg)



from __future__ import unicode_literals
from decimal import Decimal
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, HttpResponseForbidden

from .forms import ProcessPaymentForm
from ..core import BasicProvider

CENTS = Decimal('0.01')


class DotpayProvider(BasicProvider):
    '''
    dotpay.pl payment provider

    seller_id:
        seller ID, assigned by dotpay
    pin:
        PIN
    channel:
        default payment channel (consult dotpay.pl reference guide)
    lang:
        UI language
    lock:
        whether to disable channels other than the default selected above
    '''
    _method = 'post'

    def __init__(self, seller_id, pin,
                 endpoint='https://ssl.dotpay.pl/test_payment/',
                 channel=0, lang='pl', lock=False, **kwargs):
        self.seller_id = seller_id
        self.pin = pin
        self.endpoint = endpoint
        self.channel = channel
        self.lang = lang
        self.lock = lock
        super(DotpayProvider, self).__init__(**kwargs)
        if not self._capture:
            raise ImproperlyConfigured(
                'Dotpay does not support pre-authorization.')

    def get_action(self, payment):
        return self.endpoint

    def get_hidden_fields(self, payment):
        if not payment.description:
            raise ValueError('Payment description is required')

        data = {
            'id': self.seller_id,
            'amount': Decimal(str(payment.total)).quantize(CENTS),
            'control': str(payment.id),
            'currency': payment.currency,
            'description': payment.description,
            'lang': self.lang,
            'channel': str(self.channel),
            'ch_lock': '1' if self.lock else '0',
            'URL': payment.get_success_url(),
            'URLC': self.get_return_url(payment),
            'type': '1'}
        return data

    def process_data(self, payment, request):
        form = ProcessPaymentForm(payment=payment, pin=self.pin,
                                  data=request.POST or None)
        if not form.is_valid():
            return HttpResponseForbidden('FAILED')
        form.save()
        return HttpResponse('OK')



from __future__ import unicode_literals
import time

from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseForbidden, HttpResponse
import jwt

from .forms import PaymentForm, ProcessPaymentForm
from ..core import BasicProvider


class GoogleWalletProvider(BasicProvider):

    def __init__(self, seller_id, seller_secret,
                 library='https://sandbox.google.com/checkout/inapp/lib/buy.js',
                 **kwargs):
        self.seller_id = seller_id
        self.seller_secret = seller_secret
        self.library = library
        super(GoogleWalletProvider, self).__init__(**kwargs)
        if not self._capture:
            raise ImproperlyConfigured(
                'Google Wallet does not support pre-authorization.')

    def get_jwt_data(self, payment):

        current_time = int(time.time())
        exp_time = current_time + 3600

        jwt_info = {
            'iss': self.seller_id,
            'aud': 'Google',
            'typ': 'google/payments/inapp/item/v1',
            'iat': current_time,
            'exp': exp_time,
            'request': {
                'currencyCode': payment.currency,
                'price': str(payment.total),
                'name': payment.description or 'Total payment',
                'sellerData': payment.token}}

        return jwt.encode(jwt_info, self.seller_secret)

    def get_form(self, payment, data=None):
        kwargs = {
            'data': data,
            'payment': payment,
            'provider': self,
            'action': '',
            'hidden_inputs': False}
        return PaymentForm(**kwargs)

    def get_process_form(self, payment, request):
        return ProcessPaymentForm(payment=payment, provider=self,
                                  data=request.POST or None)

    def get_token_from_request(self, payment, request):
        form = self.get_process_form(payment, request)
        if form.is_valid():
            return form.token

    def process_data(self, payment, request):
        form = self.get_process_form(payment, request)
        if not form.is_valid():
            return HttpResponseForbidden('FAILED')
        form.save()
        return HttpResponse(form.order_id)



from __future__ import unicode_literals
from unittest import TestCase
from mock import patch, MagicMock, Mock

from . import AuthorizeNetProvider
from .. import RedirectNeeded


LOGIN_ID = 'abcd1234'
TRANSACTION_KEY = '1234abdd'

PROCESS_DATA = {
    'number': '4007000000027',
    'expiration_0': '5',
    'expiration_1': '2020',
    'cvv2': '123'}

STATUS_CONFIRMED = '1'


class Payment(Mock):
    id = 1
    variant = 'authorizenet'
    currency = 'USD'
    total = 100
    status = 'waiting'
    transaction_id = None
    captured_amount = 0

    def get_process_url(self):
        return 'http://example.com'

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_success_url(self):
        return 'http://success.com'

    def change_status(self, status):
        self.status = status



class TestAuthorizeNetProvider(TestCase):

    def setUp(self):
        self.payment = Payment()

    def test_provider_redirects_to_success_on_payment_success(self):
        provider = AuthorizeNetProvider(
            login_id=LOGIN_ID, transaction_key=TRANSACTION_KEY)

        response_data = [
            STATUS_CONFIRMED,
            '',
            '',
            'This transaction has been approved.',
            '',
            '',
            '1234']

        with patch('requests.post') as mocked_post:
            post = MagicMock()
            post.text = '|'.join(response_data)
            mocked_post.return_value = post
            with self.assertRaises(RedirectNeeded) as exc:
                provider.get_form(self.payment, data=PROCESS_DATA)
                self.assertEqual(exc.args[0], self.payment.get_success_url())
        self.assertEqual(self.payment.status, 'confirmed')
        self.assertEqual(self.payment.captured_amount, self.payment.total)

    def test_provider_shows_validation_error_message(self):
        provider = AuthorizeNetProvider(
            login_id=LOGIN_ID, transaction_key=TRANSACTION_KEY)

        error_msg = 'The merchant does not accept this type of credit card.'
        response_data = [
            '',
            '',
            '',
            error_msg,
            '',
            '',
            '1234']

        with patch('requests.post') as mocked_post:
            post = MagicMock()
            post.ok = False
            post.text = '|'.join(response_data)
            mocked_post.return_value = post
            form = provider.get_form(self.payment, data=PROCESS_DATA)
            self.assertEqual(form.errors['__all__'][0], error_msg)
        self.assertEqual(self.payment.status, 'error')
        self.assertEqual(self.payment.captured_amount, 0)



from __future__ import unicode_literals

from django import forms
import jwt

from ..forms import PaymentForm as BasePaymentForm
from .widgets import WalletWidget


class PaymentForm(BasePaymentForm):

    def __init__(self, *args, **kwargs):
        super(PaymentForm, self).__init__(*args, **kwargs)
        widget = WalletWidget(provider=self.provider, payment=self.payment)
        self.fields['payment'] = forms.CharField(widget=widget, required=False)


class ProcessPaymentForm(forms.Form):

    jwt = forms.CharField(required=True)

    def __init__(self, payment, provider, **kwargs):
        super(ProcessPaymentForm, self).__init__(**kwargs)
        self.provider = provider
        self.payment = payment

    def clean_jwt(self):
        payload = super(ProcessPaymentForm, self).clean().get('jwt')
        try:
            jwt_data = jwt.decode(
                payload.encode('utf-8'), self.provider.seller_secret,
                audience=self.provider.seller_id, issuer='Google')
        except (jwt.DecodeError, jwt.InvalidAudienceError, jwt.InvalidIssuer):
            raise forms.ValidationError('Incorrect response')

        self.token = jwt_data['request']['sellerData']

        if self.payment and self.payment.token != self.token:
            raise forms.ValidationError('Incorrect payment token')

        self.order_id = jwt_data['response']['orderId']
        return payload

    def save(self):
        self.payment.transaction_id = self.order_id
        self.payment.captured_amount = self.payment.total
        self.payment.change_status('confirmed')
        self.payment.save()
